#include <LiquidCrystal.h>                                                                            //
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);            //Выводы индикатора (RS,E,D4,D5,D6,D7);               //
//----------------------------------------------------------------------------------------------------//
unsigned int          interval_izm  =     1000; //Время счета входной частоты в mS.                   //
unsigned long         Fref          = 16000000; //Тактовая частота кварцевого генератора в Гц.        //
         int          F_korr        =      100; //Максимальная корректировка частоты кварца +/- Гц.   //
         boolean      Gate=0,start_gate,stop_gate;                                                    //
volatile byte         tik_int,t2,p10,p11,p12;                                                         //
volatile unsigned int ovf_int,t1;                                                                     //
unsigned int          ovf_tik_timer,t3,t4;                                                            //
         long         R_korr;                                                                         //
unsigned long         N,M;                                                                            //
         float        Fin,korr,k_M;                                                                   //
//----------------------------------------------------------------------------------------------------//
void setup()                                                                                          //
{                                                                                                     //
  lcd.begin(16, 2);                            //Задание размерности экрана 16х2                      //
  analogReference(DEFAULT);                    //Задаем работу АЦП от опорного напряжения +5в         //
  digitalWrite( 2,HIGH);                       //Включаем подтягивающий резистор                      //
  //---Грубая корректировка частоты кварца------------------------------------------------------------//
  digitalWrite(10,HIGH);digitalWrite(11,HIGH);digitalWrite(12,HIGH);                                  //
  if (digitalRead(10)==0) p10=1; else p10=0;                                                          //
  if (digitalRead(11)==0) p11=1; else p11=0;                                                          //
  if (digitalRead(12)==0) p12=1; else p12=0;                                                          //
  R_korr=-(4*F_korr+2*F_korr+1*F_korr)/1.0+4*p10*F_korr+2*p11*F_korr+1*p12*F_korr;                    //
  lcd.clear();lcd.setCursor(0,0);lcd.print("F korr =");lcd.print(R_korr);lcd.print("Hz");delay(2000); //
  //---Настройка прерывания int 0---------------------------------------------------------------------//
  EICRA   = (1<<ISC01) | (1<<ISC00);           //настройка прерывания  int 0 по фронту импульса       //
  //---Настройка таймера 0----------------------------------------------------------------------------//
  TCCR0B=0;TIMSK0=0;                           //Отключаем таймер 0                                   //
  //---Настройка таймера 1----------------------------------------------------------------------------//
  TCCR1A  = 0; TCCR1B  = 0; TCNT1   = 0;                                                              //
  OCR1A   = 15999;                             //отсчитать 16 000 импульсов до прерывания таймера     //
  TIMSK1 |= (1 << OCIE1A);                     //Настраиваем регистр масок прерывания                 //
  TCCR1B  = (1<<CS10)|(1<<WGM12);              //Запускаем Таймер 1                                   //
}                                                                                                     //
//---Прерывание по int 0------------------------------------------------------------------------------//
ISR (INT0_vect)                                                                                       //
{                                                                                                     //
  tik_int=tik_int+1;                                                                                  //
  if (Gate==1 && tik_int>=255) {ovf_int=ovf_int+1;tik_int=0;                             }            //
  if (Gate==1 && start_gate==1){TCNT1=0;ovf_tik_timer=0;tik_int=0;ovf_int=0;start_gate=0;}            //
  if (Gate==1 &&  stop_gate==1){t4=TCNT1;t3=ovf_tik_timer;t2=tik_int;t1=ovf_int;Gate=0;  }            //
}                                                                                                     //
//---Прерывание по таймеру 1--------------------------------------------------------------------------//
ISR(TIMER1_COMPA_vect)                         //Обработка прерывания по совпадению таймера           //
{                                                                                                     //
  ovf_tik_timer=ovf_tik_timer+1;               //Считываем количество переполнений таймера            //
  if (ovf_tik_timer>=interval_izm && Gate==1 && start_gate==0)stop_gate=1;//Окончание измерения       //
}                                                                                                     //
//----------------------------------------------------------------------------------------------------//
void loop()                                                                                           //
{                                                                                                     //
  //---Условия окончания измерения по времени при низкой частоте--------------------------------------//
  if (ovf_tik_timer>=2.0*interval_izm && Gate==1) {t1=0;t2=0;t3=ovf_tik_timer;t4=TCNT1;Gate=0;}       //
  //---Начало режима индикации------------------------------------------------------------------------//
  if (Gate==0)                                                                                        //
  {                                                                                                   //
    EIMSK=(0<<INT0);                           //запрет прерываний int 0                              //
    //---Точная корректировка частоты кварца----------------------------------------------------------//
    korr=(float)-F_korr+((2.0*F_korr*analogRead(0))/1024.0);                                          //
    k_M =(float)(Fref+korr+R_korr)/16000000;                                                          //
    //---Расчет частоты-------------------------------------------------------------------------------//
    N=(unsigned long)t1*255+t2;                //Количество входных импульсов                         //
    M=(unsigned long)t3*16000+t4;                                                                     //
    M=(unsigned long)M*k_M;                    //Количество импульсов опорной частоты                 //
    Fin=(float)(1.0*16000000/M)*N;             //Измеренная частота                                   //
    //---Начало вывода показаний на экран-------------------------------------------------------------//
    lcd.clear();lcd.setCursor(0,0);lcd.print(N);lcd.setCursor(8,0);lcd.print(M);lcd.setCursor(0,1);   //
    if (Fin>=      0.0 && Fin<      10.0){lcd.print("     ");lcd.print(Fin,6);lcd.print(" Hz");}      //
    if (Fin>=     10.0 && Fin<     100.0){lcd.print("    " );lcd.print(Fin,5);lcd.print("  Hz");}     //
    if (Fin>=    100.0 && Fin<    1000.0){lcd.print("   "  );lcd.print(Fin,4);lcd.print("   Hz");}    //
    if (Fin>=   1000.0 && Fin<   10000.0){lcd.print("  "   );lcd.print(Fin,3);lcd.print("    Hz");}   //
    if (Fin>=  10000.0 && Fin<  100000.0){lcd.print(" "    );lcd.print(Fin,2);lcd.print("     Hz");}  //
    if (Fin>= 100000.0 && Fin< 1000000.0){lcd.print(""     );lcd.print(Fin,1);lcd.print("      Hz");} //
    if (Fin>=1000000.0 && Fin<10000000.0){lcd.print(""     );lcd.print(Fin,0);lcd.print("       Hz");}//
    //---Конец вывода показаний на экран--------------------------------------------------------------//
    EIMSK=(1<<INT0);                           //разрешение прерываний int 0                          //
    ovf_tik_timer=0;start_gate=1;stop_gate=0;Gate=1;                                                  //
  }                                                                                                   //
  //---Конец режима индикации-------------------------------------------------------------------------//
}                                                                                                     //
